import 'dart:math';

import 'package:flutter/material.dart';

///
/// @dependencies flutter_star: ^0.1.2
/// @date 2020-05-21
///

/// 星星评分组件
class StarScore extends StatelessWidget {
  final Star star;
  final double score;
  final Widget tail;
  final int max;
  final double contentPadding;
  final double spacing;

  StarScore({
    Key key,
    this.score,
    this.tail,
    this.max,
    this.contentPadding = 0,
    this.spacing = 0,
    Color fillColor = Colors.yellow,
    Color emptyColor = Colors.grey,
    double size,
    double fat = 0.5,
    int cornerNum = 5,
  })  : assert(score <= max),
        this.star = Star(
          emptyColor: emptyColor,
          fillColor: fillColor,
          size: size,
          fat: fat,
          num: cornerNum,
        ),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    List<Widget> li = [];
    int count = score.floor();
    for (int i = 0; i < count; i++) {
      // 判断是不是第一个星星
      bool isFirst = i == 0;
      li.add(_buildStarWidget(star.copyWith(progress: 1.0), isFirst: isFirst));
    }
    if (score != max.toDouble()) {
      // 判断是不是第一个星星
      bool isFirst = count == 0;
      // 不满星
      li.add(_buildStarWidget(star.copyWith(progress: score - count),
          isFirst: isFirst));
    }

    // 空星
    var empty = max - count - 1;
    for (int i = 0; i < empty; i++) {
      li.add(_buildStarWidget(star.copyWith(progress: 0)));
    }
    return Wrap(
      crossAxisAlignment: WrapCrossAlignment.center,
      children: _buildItems(li),
    );
  }

  Widget _buildStarWidget(Star star, {bool isFirst = false}) => Padding(
        padding: EdgeInsets.only(left: isFirst ? 0 : spacing),
        child: StarWidget(star: star),
      );

  List<Widget> _buildItems(List<Widget> li) {
    List<Widget> items = [];
    // 星星评分组件
    items.addAll(li);
    // 与尾部控件间距
    items.add(SizedBox(width: contentPadding));
    // 尾部控件
    if (tail != null) {
      items.add(tail);
    }
    return items;
  }
}

/// 自定义评分组件
class CustomRating extends StatefulWidget {
  final int max;
  final Star star;
  final double score;
  final Function(double) onRating;

  CustomRating(
      {Key key,
      this.max = 5,
      this.score = 0,
      this.star = const Star(),
      @required this.onRating})
      : assert(score <= max),
        super(key: key);

  @override
  _CustomRatingState createState() => _CustomRatingState();
}

/// 自定义评分组件状态
class _CustomRatingState extends State<CustomRating> {
  double _score;

  @override
  void initState() {
    _score = widget.score;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    var li = <StarWidget>[];

    int count = _score.floor(); //满星
    for (int i = 0; i < count; i++) {
      li.add(StarWidget(star: widget.star.copyWith(progress: 1.0)));
    }

    if (_score != widget.max.toDouble())
      li.add(StarWidget(
          star: widget.star.copyWith(progress: _score - count))); //不满星

    var empty = widget.max - count - 1; // 空星
    for (int i = 0; i < empty; i++) {
      li.add(StarWidget(star: widget.star.copyWith(progress: 0)));
    }
    return GestureDetector(
      onTapDown: (d) {
        setState(() {
          _score = d.localPosition.dx / widget.star.size;
          if (_score - _score.floor() > 0.5) {
            _score = _score.floor() + 1.0;
          } else {
            _score = _score.floor() + 0.5;
          }

          if (_score >= widget.max.toDouble()) {
            _score = widget.max.toDouble();
          }
          widget.onRating(_score);
        });
      },
      child: Wrap(
        children: li,
      ),
    );
  }
}

/// 星星属性
class Star {
  final int num;
  final double progress;
  final Color emptyColor;
  final Color fillColor;
  final double size;
  final double fat;

  const Star(
      {this.progress = 0,
      this.fat = 0.5,
      this.fillColor = Colors.yellow,
      this.emptyColor = Colors.grey,
      this.num = 5,
      this.size = 25});

  Star copyWith(
          {double progress,
          double fat,
          Color fillColor,
          Color emptyColor,
          int num,
          double size}) =>
      Star(
        progress: progress ?? this.progress,
        fat: fat ?? this.fat,
        fillColor: fillColor ?? this.fillColor,
        emptyColor: emptyColor ?? this.emptyColor,
        num: num ?? this.num,
        size: size ?? this.size,
      );
}

/// 星星组件
class StarWidget extends StatelessWidget {
  final Star star;

  StarWidget({Key key, this.star = const Star()}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      width: star.size,
      height: star.size,
      child: CustomPaint(
        painter: _StarPainter(star),
      ),
    );
  }
}

/// 星星绘制
class _StarPainter extends CustomPainter {
  Star star;
  Paint _paint;
  Paint _filePaint;
  Path _path;
  double _radius;

  _StarPainter(this.star) {
    _paint = Paint()
      ..color = (star.emptyColor)
      ..isAntiAlias = true;

    _filePaint = Paint()..color = (star.fillColor);
    _path = Path();
    _radius = star.size / 2.0;
    nStarPath(star.num, _radius, _radius * star.fat);
  }

  @override
  void paint(Canvas canvas, Size size) {
    canvas.translate(_radius, _radius);
    canvas.drawPath(_path, _paint);
    canvas.clipRect(Rect.fromLTRB(
        -_radius, -_radius, _radius * 2 * star.progress - _radius, _radius));
    canvas.drawPath(_path, _filePaint);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return true;
  }

  Path nStarPath(int num, double R, double r, {dx = 0, dy = 0, rotate = 0}) {
    // 重置路径
    _path.reset();
    // 每份的角度
    double perRad = 2 * pi / num;
    // a角
    double radA = perRad / 2 / 2 + rotate;
    // 起始b角
    double radB = 2 * pi / (num - 1) / 2 - radA / 2 + radA + rotate;
    // 移动到起点
    _path.moveTo(cos(radA) * R + dx, -sin(radA) * R + dy);
    for (int i = 0; i < num; i++) {
      // 循环生成点，路径连至
      _path.lineTo(
          cos(radA + perRad * i) * R + dx, -sin(radA + perRad * i) * R + dy);
      _path.lineTo(
          cos(radB + perRad * i) * r + dx, -sin(radB + perRad * i) * r + dy);
    }
    _path.close();
    return _path;
  }
}
